@@ -6,8 +6,9 @@
 #' @noRd
 #' @return to be documented
 aicc_f <-
-  function(e,ts,n){
+  function(e,ts,n,pena=1){
     n=length(e)
     #n*log(sum(e^2)/n)+n*log(2*pi)+n*(n+ts)/(n-2-ts)
-    n*log(sum(e^2)/n)+n*log(2*pi)+n*(n+ts)/(n-1-ts)
+    n*log(sum(e^2)/n)+n*log(2*pi)+n*(n+pena*ts)/(n-1-pena*ts)
+
   }
